@@ -1,4 +1,7 @@
+import pandas as pd
+
 from kafi.functional import Functional
+from kafi.helpers import explode_normalize
 
 # Constants
 
@@ -8,11 +11,12 @@ ALL_MESSAGES = -1
 
 class Pandas(Functional):
     def to_df(self, topic, n=ALL_MESSAGES, **kwargs):
-        import pandas as pd
         #
-
         def foldl_function(acc, message_dict):
-            df = pd.DataFrame.from_records([message_dict["value"]])
+            # df = pd.DataFrame.from_records([message_dict["value"]])
+            df = pd.json_normalize(message_dict["value"])
+            if "explode" in kwargs and kwargs["explode"] == True:
+                df = explode_normalize(df)
             #
             acc = pd.concat([acc, df], ignore_index=True)
             #
@@ -26,7 +30,6 @@ class Pandas(Functional):
     def from_df(self, df, topic, n=ALL_MESSAGES, **kwargs):
         n_int = n
         #
-
         producer = self.producer(topic, **kwargs)
         counter_int = 0
         for _, row in df.iterrows():
@@ -34,7 +37,14 @@ class Pandas(Functional):
                 if counter_int >= n_int:
                     break
             #
-            producer.produce(row.to_dict())
+            d = row.to_dict()
+            delete_key_str_list = []
+            for key_str, value in d.items():
+                if pd.isna(value):
+                    delete_key_str_list.append(key_str)
+            for delete_key_str in delete_key_str_list:
+                del d[delete_key_str]
+            producer.produce(d)
             #
             counter_int += 1
         producer.close()
