@@ -15,6 +15,9 @@ from kafi.helpers import *
 #
 
 OFFSET_INVALID = -1001
+OFFSET_END = -1
+
+#
 
 class TestSingleStorageBase(unittest.TestCase):
     def setUp(self):
@@ -43,6 +46,8 @@ class TestSingleStorageBase(unittest.TestCase):
         self.headers_str_str_tuple_list = [("header_field1", "header_value1"), ("header_field2", "header_value2")]
         self.headers_str_str_dict = {"header_field1": "header_value1", "header_field2": "header_value2"}
         #
+        self.nested_json_str_list = [{"state": "Florida", "shortname": "FL", "info": {"governor": "Rick Scott"}, "counties": [{"name": "Dade", "population": 12345}, {"name": "Broward", "population": 40000}, {"name": "Palm Beach", "population": 60000}]}, {"state": "Ohio", "shortname": "OH", "info": {"governor": "John Kasich"}, "counties": [{"name": "Summit", "population": 1234}, {"name": "Cuyahoga", "population": 1337}]}]
+        #
         self.topic_str_list = []
         self.group_str_list = []
         #
@@ -728,6 +733,44 @@ class TestSingleStorageBase(unittest.TestCase):
         config_str_config_dict_dict = s.configs(verbose=True)
         self.assertIn("local", config_str_config_dict_dict)
 
+    def test_delete_records(self):
+        if self.__class__.__name__ == "TestSingleStorageBase":
+            return
+        #
+        s = self.get_storage()
+        #
+        if s.__class__.__name__ == "Cluster":
+            topic_str = self.create_test_topic_name()
+            s.create(topic_str)
+            w = s.producer(topic_str, value_type="str")
+            w.produce(self.snack_str_list)
+            w.close()
+            #
+            n_int = s.l(topic_str)[topic_str]
+            self.assertEqual(n_int, 3)
+            #
+            offsets = topic_str
+            s.delete_records(offsets)
+            time.sleep(1)
+            n_int = s.l(topic_str)[topic_str]
+            self.assertEqual(n_int, 0)
+            ###
+            topic_str = self.create_test_topic_name()
+            s.create(topic_str)
+            w = s.producer(topic_str, value_type="str")
+            w.produce(self.snack_str_list)
+            w.close()
+            #
+            n_int = s.l(topic_str)[topic_str]
+            self.assertEqual(n_int, 3)
+            #
+            offsets = {topic_str: {0: 2}}
+            s.delete_records(offsets)
+            i = 0
+            time.sleep(1)
+            n_int = s.l(topic_str)[topic_str]
+            self.assertEqual(n_int, 1)
+
     # Shell
 
     # Shell.cat -> Functional.map -> Functional.flatmap -> Functional.foldl -> ClusterConsumer.consumer/KafkaConsumer.foldl/ClusterConsumer.close -> ClusterConsumer.consume
@@ -825,6 +868,8 @@ class TestSingleStorageBase(unittest.TestCase):
         s.create(topic_str1)
         producer = s.producer(topic_str1, value_type="json")
         producer.produce(self.snack_bytes_list, headers=self.headers_str_bytes_tuple_list)
+        producer.produce(self.snack_bytes_list, headers=self.headers_str_bytes_tuple_list)
+        producer.produce(self.snack_bytes_list, headers=self.headers_str_bytes_tuple_list)
         producer.close()
         #
         topic_str2 = self.create_test_topic_name()
@@ -845,6 +890,14 @@ class TestSingleStorageBase(unittest.TestCase):
             self.assertEqual(1, len(message_dict_list2))
             self.assertEqual(message_dict_list2[0]["value"], self.snack_ish_dict_list[0])
             self.assertEqual(message_dict_list2[0]["headers"], self.headers_str_bytes_tuple_list)
+        # test that the consumer does not consume too many messages (the former bug occurred when: n % consumer_batch_size > 0, n > consumer_batch_size * 2, source topic length > n (e.g. source topic length = 100000, n = 42700, consumer_batch_size = 1000))
+        topic_str3 = self.create_test_topic_name()
+        s.create(topic_str3)
+        #
+        group_str2 = self.create_test_group_name()
+        (consume_n_int1, written_n_int1) = s.cp(topic_str1, s, topic_str3, group=group_str2, source_type="json", target_type="json", consume_batch_size=3, map_function=map_ish, n=7)
+        self.assertEqual(7, consume_n_int1)
+        self.assertEqual(7, written_n_int1)
 
     def test_wc(self):
         if self.__class__.__name__ == "TestSingleStorageBase":
@@ -1005,6 +1058,40 @@ class TestSingleStorageBase(unittest.TestCase):
         self.assertEqual(260.0, message_dict_list[1]["value"]["calories"])
         self.assertEqual(80.0, message_dict_list[2]["value"]["calories"])
 
+    def test_explode(self):
+        if self.__class__.__name__ == "TestSingleStorageBase":
+            return
+        #
+        s = self.get_storage()
+        #
+        topic_str1 = self.create_test_topic_name()
+        s.create(topic_str1)
+        producer = s.producer(topic_str1, type="json")
+        producer.produce(self.nested_json_str_list)
+        producer.close()
+        #
+        group_str1 = self.create_test_group_name()
+        df1 = s.to_df(topic_str1, group=group_str1, type="json")
+        self.assertTrue(len(df1), 2)
+        self.assertTrue(df1.iloc[0]["state"], "Florida")
+        self.assertTrue(df1.iloc[1]["state"], "Ohio")
+        self.assertTrue(isinstance(df1.iloc[0]["counties"], list))
+        self.assertTrue(isinstance(df1.iloc[1]["counties"], list))
+        #
+        group_str2 = self.create_test_group_name()
+        df2 = s.to_df(topic_str1, group=group_str2, type="json", explode=True)
+        self.assertTrue(len(df2), 5)
+        self.assertTrue(df2.iloc[0]["state"], "Florida")
+        self.assertTrue(df2.iloc[1]["state"], "Florida")
+        self.assertTrue(df2.iloc[2]["state"], "Florida")
+        self.assertTrue(df2.iloc[3]["state"], "Ohio")
+        self.assertTrue(df2.iloc[4]["state"], "Ohio")
+        self.assertTrue(df2.iloc[0]["info.governor"], "Rick Scott")
+        self.assertTrue(df2.iloc[1]["counties.name"], "Broward")
+        self.assertTrue(df2.iloc[2]["counties.population"], "60000")
+        self.assertTrue(df2.iloc[3]["shortname"], "OH")
+        self.assertTrue(df2.iloc[4]["counties.population"], "1337")
+
     # Add-Ons
 
     def test_recreate(self):
